@@ -1,6 +1,8 @@
 use birli::{
-    context_to_jones_array, correct_cable_lengths, correct_geometry, get_flaggable_timesteps,
-    init_flag_array, io::write_uvfits,
+    context_to_jones_array, correct_cable_lengths, correct_geometry,
+    flags::{expand_flag_array, flag_to_weight_array, get_weight_factor},
+    get_flaggable_timesteps,
+    io::write_uvfits,
 };
 use criterion::{black_box, criterion_group, criterion_main, Criterion};
 use glob::glob;
@@ -62,7 +64,7 @@ fn get_context_1254670392_avg() -> CorrelatorContext {
     let test_path = Path::new(&test_dir);
     let vis_path = test_path.join("1254670392_vis");
     let metafits_path = vis_path
-        .join("1254670392.metafits")
+        .join("1254670392.fixed.metafits")
         .to_str()
         .unwrap()
         .to_owned();
@@ -85,7 +87,6 @@ fn bench_context_to_jones_array_mwax_half_1247842824(crt: &mut Criterion) {
     let timestep_range = timestep_idxs[0]..timestep_idxs[timestep_idxs.len() - 1] + 1;
     let coarse_chan_idxs = context.common_coarse_chan_indices.clone();
     let coarse_chan_range = coarse_chan_idxs[0]..coarse_chan_idxs[coarse_chan_idxs.len() - 1] + 1;
-    // let img_baseline_idxs: Vec<usize> = (0..context.metafits_context.num_baselines).collect();
     crt.bench_function("context_to_jones_array - mwax_half_1247842824", |bch| {
         bch.iter(|| {
             context_to_jones_array(
@@ -104,7 +105,6 @@ fn bench_context_to_jones_array_ord_half_1196175296(crt: &mut Criterion) {
     let timestep_range = timestep_idxs[0]..timestep_idxs[timestep_idxs.len() - 1] + 1;
     let coarse_chan_idxs = context.common_coarse_chan_indices.clone();
     let coarse_chan_range = coarse_chan_idxs[0]..coarse_chan_idxs[coarse_chan_idxs.len() - 1] + 1;
-    // let img_baseline_idxs: Vec<usize> = (0..context.metafits_context.num_baselines).collect();
     crt.bench_function("context_to_jones_array - ord_half_1196175296", |bch| {
         bch.iter(|| {
             context_to_jones_array(
@@ -120,23 +120,24 @@ fn bench_context_to_jones_array_ord_half_1196175296(crt: &mut Criterion) {
 fn bench_correct_cable_lengths_mwax_half_1247842824(crt: &mut Criterion) {
     let context = get_context_mwax_half_1247842824();
 
-    let img_timestep_idxs = get_flaggable_timesteps(&context).unwrap();
+    let sel_timestep_idxs = get_flaggable_timesteps(&context).unwrap();
 
-    let img_timestep_range =
-        *img_timestep_idxs.first().unwrap()..(*img_timestep_idxs.last().unwrap() + 1);
-    let img_coarse_chan_idxs = &context.common_coarse_chan_indices;
-    let img_coarse_chan_range =
-        *img_coarse_chan_idxs.first().unwrap()..(*img_coarse_chan_idxs.last().unwrap() + 1);
+    let sel_timestep_range =
+        *sel_timestep_idxs.first().unwrap()..(*sel_timestep_idxs.last().unwrap() + 1);
+    let sel_coarse_chan_idxs = &context.common_coarse_chan_indices;
+    let sel_coarse_chan_range =
+        *sel_coarse_chan_idxs.first().unwrap()..(*sel_coarse_chan_idxs.last().unwrap() + 1);
 
     let (mut jones_array, _) =
-        context_to_jones_array(&context, &img_timestep_range, &img_coarse_chan_range, None);
+        context_to_jones_array(&context, &sel_timestep_range, &sel_coarse_chan_range, None)
+            .unwrap();
 
     crt.bench_function("correct_cable_lengths - mwax_half_1247842824", |bch| {
         bch.iter(|| {
             correct_cable_lengths(
                 black_box(&context),
                 black_box(&mut jones_array),
-                black_box(&img_coarse_chan_range),
+                black_box(&sel_coarse_chan_range),
             )
         })
     });
@@ -145,22 +146,23 @@ fn bench_correct_cable_lengths_mwax_half_1247842824(crt: &mut Criterion) {
 fn bench_correct_cable_lengths_ord_half_1196175296(crt: &mut Criterion) {
     let context = get_context_ord_half_1196175296();
 
-    let img_timestep_idxs = get_flaggable_timesteps(&context).unwrap();
+    let sel_timestep_idxs = get_flaggable_timesteps(&context).unwrap();
 
-    let img_timestep_range =
-        *img_timestep_idxs.first().unwrap()..(*img_timestep_idxs.last().unwrap() + 1);
-    let img_coarse_chan_idxs = &context.common_coarse_chan_indices;
-    let img_coarse_chan_range =
-        *img_coarse_chan_idxs.first().unwrap()..(*img_coarse_chan_idxs.last().unwrap() + 1);
+    let sel_timestep_range =
+        *sel_timestep_idxs.first().unwrap()..(*sel_timestep_idxs.last().unwrap() + 1);
+    let sel_coarse_chan_idxs = &context.common_coarse_chan_indices;
+    let sel_coarse_chan_range =
+        *sel_coarse_chan_idxs.first().unwrap()..(*sel_coarse_chan_idxs.last().unwrap() + 1);
 
     let (mut jones_array, _) =
-        context_to_jones_array(&context, &img_timestep_range, &img_coarse_chan_range, None);
+        context_to_jones_array(&context, &sel_timestep_range, &sel_coarse_chan_range, None)
+            .unwrap();
     crt.bench_function("correct_cable_lengths - ord_half_1196175296", |bch| {
         bch.iter(|| {
             correct_cable_lengths(
                 black_box(&context),
                 black_box(&mut jones_array),
-                black_box(&img_coarse_chan_range),
+                black_box(&sel_coarse_chan_range),
             )
         })
     });
@@ -172,19 +174,21 @@ fn bench_correct_geometry_mwax_half_1247842824(crt: &mut Criterion) {
 
     let img_timestep_range =
         *img_timestep_idxs.first().unwrap()..(*img_timestep_idxs.last().unwrap() + 1);
-    let img_coarse_chan_idxs = &context.common_coarse_chan_indices;
-    let img_coarse_chan_range =
-        *img_coarse_chan_idxs.first().unwrap()..(*img_coarse_chan_idxs.last().unwrap() + 1);
+    let sel_coarse_chan_idxs = &context.common_coarse_chan_indices;
+    let sel_coarse_chan_range =
+        *sel_coarse_chan_idxs.first().unwrap()..(*sel_coarse_chan_idxs.last().unwrap() + 1);
 
     let (mut jones_array, _) =
-        context_to_jones_array(&context, &img_timestep_range, &img_coarse_chan_range, None);
+        context_to_jones_array(&context, &img_timestep_range, &sel_coarse_chan_range, None)
+            .unwrap();
     crt.bench_function("correct_geometry - mwax_half_1247842824", |bch| {
         bch.iter(|| {
             correct_geometry(
                 black_box(&context),
                 black_box(&mut jones_array),
                 black_box(&img_timestep_range),
-                black_box(&img_coarse_chan_range),
+                black_box(&sel_coarse_chan_range),
+                None,
                 None,
             );
         })
@@ -197,19 +201,21 @@ fn bench_correct_geometry_ord_half_1196175296(crt: &mut Criterion) {
 
     let img_timestep_range =
         *img_timestep_idxs.first().unwrap()..(*img_timestep_idxs.last().unwrap() + 1);
-    let img_coarse_chan_idxs = &context.common_coarse_chan_indices;
-    let img_coarse_chan_range =
-        *img_coarse_chan_idxs.first().unwrap()..(*img_coarse_chan_idxs.last().unwrap() + 1);
+    let sel_coarse_chan_idxs = &context.common_coarse_chan_indices;
+    let sel_coarse_chan_range =
+        *sel_coarse_chan_idxs.first().unwrap()..(*sel_coarse_chan_idxs.last().unwrap() + 1);
 
     let (mut jones_array, _) =
-        context_to_jones_array(&context, &img_timestep_range, &img_coarse_chan_range, None);
+        context_to_jones_array(&context, &img_timestep_range, &sel_coarse_chan_range, None)
+            .unwrap();
     crt.bench_function("correct_geometry - ord_half_1196175296", |bch| {
         bch.iter(|| {
             correct_geometry(
                 black_box(&context),
                 black_box(&mut jones_array),
                 black_box(&img_timestep_range),
-                black_box(&img_coarse_chan_range),
+                black_box(&sel_coarse_chan_range),
+                None,
                 None,
             );
         })
@@ -222,31 +228,38 @@ fn bench_uvfits_output_1254670392_avg_none(crt: &mut Criterion) {
 
     let img_timestep_range =
         *img_timestep_idxs.first().unwrap()..(*img_timestep_idxs.last().unwrap() + 1);
-    let img_coarse_chan_idxs = &context.common_coarse_chan_indices;
-    let img_coarse_chan_range =
-        *img_coarse_chan_idxs.first().unwrap()..(*img_coarse_chan_idxs.last().unwrap() + 1);
+    let sel_coarse_chan_idxs = &context.common_coarse_chan_indices;
+    let sel_coarse_chan_range =
+        *sel_coarse_chan_idxs.first().unwrap()..(*sel_coarse_chan_idxs.last().unwrap() + 1);
 
     let img_baseline_idxs: Vec<usize> = (0..context.metafits_context.num_baselines).collect();
 
     let (jones_array, flag_array) =
-        context_to_jones_array(&context, &img_timestep_range, &img_coarse_chan_range, None);
+        context_to_jones_array(&context, &img_timestep_range, &sel_coarse_chan_range, None)
+            .unwrap();
 
     let tmp_dir = tempdir().unwrap();
     let uvfits_path = tmp_dir.path().join("1254670392.none.uvfits");
 
-    init_flag_array(&context, &img_timestep_range, &img_coarse_chan_range, None);
+    let weight_factor = get_weight_factor(&context);
+    let flag_array = expand_flag_array(flag_array.view(), 4);
+    let weight_array = flag_to_weight_array(flag_array.view(), weight_factor);
 
     crt.bench_function("uvfits_output - 1254670392_avg", |bch| {
         bch.iter(|| {
             write_uvfits(
                 black_box(uvfits_path.as_path()),
                 black_box(&context),
-                black_box(&jones_array),
-                black_box(&flag_array),
+                black_box(jones_array.view()),
+                black_box(weight_array.view()),
+                black_box(flag_array.view()),
                 black_box(&img_timestep_range),
-                black_box(&img_coarse_chan_range),
+                black_box(&sel_coarse_chan_range),
                 black_box(&img_baseline_idxs),
                 None,
+                None,
+                1,
+                1,
             )
             .unwrap();
         });
@@ -259,31 +272,38 @@ fn bench_uvfits_output_ord_half_1196175296_none(crt: &mut Criterion) {
 
     let img_timestep_range =
         *img_timestep_idxs.first().unwrap()..(*img_timestep_idxs.last().unwrap() + 1);
-    let img_coarse_chan_idxs = &context.common_coarse_chan_indices;
-    let img_coarse_chan_range =
-        *img_coarse_chan_idxs.first().unwrap()..(*img_coarse_chan_idxs.last().unwrap() + 1);
+    let sel_coarse_chan_idxs = &context.common_coarse_chan_indices;
+    let sel_coarse_chan_range =
+        *sel_coarse_chan_idxs.first().unwrap()..(*sel_coarse_chan_idxs.last().unwrap() + 1);
 
     let img_baseline_idxs: Vec<usize> = (0..context.metafits_context.num_baselines).collect();
 
     let (jones_array, flag_array) =
-        context_to_jones_array(&context, &img_timestep_range, &img_coarse_chan_range, None);
+        context_to_jones_array(&context, &img_timestep_range, &sel_coarse_chan_range, None)
+            .unwrap();
 
     let tmp_dir = tempdir().unwrap();
     let uvfits_path = tmp_dir.path().join("1196175296.none.uvfits");
 
-    init_flag_array(&context, &img_timestep_range, &img_coarse_chan_range, None);
+    let weight_factor = get_weight_factor(&context);
+    let flag_array = expand_flag_array(flag_array.view(), 4);
+    let weight_array = flag_to_weight_array(flag_array.view(), weight_factor);
 
     crt.bench_function("uvfits_output - ord_half_1196175296", |bch| {
         bch.iter(|| {
             write_uvfits(
                 black_box(uvfits_path.as_path()),
                 black_box(&context),
-                black_box(&jones_array),
-                black_box(&flag_array),
+                black_box(jones_array.view()),
+                black_box(weight_array.view()),
+                black_box(flag_array.view()),
                 black_box(&img_timestep_range),
-                black_box(&img_coarse_chan_range),
+                black_box(&sel_coarse_chan_range),
                 black_box(&img_baseline_idxs),
                 None,
+                None,
+                1,
+                1,
             )
             .unwrap();
         });
@@ -296,31 +316,38 @@ fn bench_uvfits_output_mwax_half_1247842824_none(crt: &mut Criterion) {
 
     let img_timestep_range =
         *img_timestep_idxs.first().unwrap()..(*img_timestep_idxs.last().unwrap() + 1);
-    let img_coarse_chan_idxs = &context.common_coarse_chan_indices;
-    let img_coarse_chan_range =
-        *img_coarse_chan_idxs.first().unwrap()..(*img_coarse_chan_idxs.last().unwrap() + 1);
+    let sel_coarse_chan_idxs = &context.common_coarse_chan_indices;
+    let sel_coarse_chan_range =
+        *sel_coarse_chan_idxs.first().unwrap()..(*sel_coarse_chan_idxs.last().unwrap() + 1);
 
     let img_baseline_idxs: Vec<usize> = (0..context.metafits_context.num_baselines).collect();
 
     let (jones_array, flag_array) =
-        context_to_jones_array(&context, &img_timestep_range, &img_coarse_chan_range, None);
+        context_to_jones_array(&context, &img_timestep_range, &sel_coarse_chan_range, None)
+            .unwrap();
 
     let tmp_dir = tempdir().unwrap();
     let uvfits_path = tmp_dir.path().join("1247842824.none.uvfits");
 
-    init_flag_array(&context, &img_timestep_range, &img_coarse_chan_range, None);
+    let weight_factor = get_weight_factor(&context);
+    let flag_array = expand_flag_array(flag_array.view(), 4);
+    let weight_array = flag_to_weight_array(flag_array.view(), weight_factor);
 
     crt.bench_function("uvfits_output - mwax_half_1247842824", |bch| {
         bch.iter(|| {
             write_uvfits(
                 black_box(uvfits_path.as_path()),
                 black_box(&context),
-                black_box(&jones_array),
-                black_box(&flag_array),
+                black_box(jones_array.view()),
+                black_box(weight_array.view()),
+                black_box(flag_array.view()),
                 black_box(&img_timestep_range),
-                black_box(&img_coarse_chan_range),
+                black_box(&sel_coarse_chan_range),
                 black_box(&img_baseline_idxs),
                 None,
+                None,
+                1,
+                1,
             )
             .unwrap();
         });
