@@ -54,7 +54,7 @@ use std::{f64::consts::PI, ops::Range};
 ///     &img_timestep_range,
 ///     &img_coarse_chan_range,
 ///     None
-/// );
+/// ).unwrap();
 ///
 /// correct_cable_lengths(&context, &mut jones_array, &img_coarse_chan_range);
 /// ```
@@ -186,7 +186,7 @@ pub fn correct_cable_lengths(
 ///     &img_timestep_range,
 ///     &img_coarse_chan_range,
 ///     None
-/// );
+/// ).unwrap();
 ///
 /// correct_geometry(
 ///     &context,
@@ -194,6 +194,7 @@ pub fn correct_cable_lengths(
 ///     &img_timestep_range,
 ///     &img_coarse_chan_range,
 ///     None,
+///     None,
 /// );
 /// ```
 pub fn correct_geometry(
@@ -204,6 +205,7 @@ pub fn correct_geometry(
     // TODO: allow subset of baselines
     // mwalib_baseline_idxs: &[usize],
     array_pos: Option<LatLngHeight>,
+    phase_centre_ra: Option<RADec>,
 ) {
     trace!("start correct_geometry");
 
@@ -228,7 +230,10 @@ pub fn correct_geometry(
 
     let integration_time_s = context.metafits_context.corr_int_time_ms as f64 / 1000.0;
 
-    let phase_centre_ra = RADec::from_mwalib_phase_or_pointing(&context.metafits_context);
+    let phase_centre_ra = match phase_centre_ra {
+        Some(pc) => pc,
+        None => RADec::from_mwalib_phase_or_pointing(&context.metafits_context),
+    };
     let tiles_xyz_geod = XyzGeodetic::get_tiles(&context.metafits_context, array_pos.latitude_rad);
 
     // Create a progress bar to show the status of the correction
@@ -290,12 +295,14 @@ mod tests {
 
     use super::{correct_cable_lengths, correct_geometry, VEL_C};
     use marlu::{
-        approx::assert_abs_diff_eq, mwalib::CorrelatorContext, precession::precess_time, time,
-        Complex, Jones, LatLngHeight, RADec, XyzGeodetic, UVW,
+        mwalib::CorrelatorContext, precession::precess_time, time, Complex, Jones, LatLngHeight,
+        RADec, XyzGeodetic, UVW,
     };
     use std::f64::consts::PI;
 
-    use crate::{context_to_jones_array, get_flaggable_timesteps, TestJones};
+    use crate::{
+        approx::assert_abs_diff_eq, context_to_jones_array, get_flaggable_timesteps, TestJones,
+    };
 
     // TODO: Why does clippy think CxxImageSet.ImageBuffer() is &[f64]?
     // TODO: deduplicate from lib.rs
@@ -337,13 +344,9 @@ mod tests {
 
         // let img_baseline_idxs: Vec<usize> = (0..context.metafits_context.num_baselines).collect();
 
-        let (jones_array, _) = context_to_jones_array(
-            &context,
-            &img_timestep_range,
-            &img_coarse_chan_range,
-            // img_baseline_idxs.as_slice(),
-            None,
-        );
+        let (jones_array, _) =
+            context_to_jones_array(&context, &img_timestep_range, &img_coarse_chan_range, None)
+                .unwrap();
 
         let jones_array = jones_array.mapv(TestJones::from);
 
@@ -520,13 +523,9 @@ mod tests {
 
         // let img_baseline_idxs: Vec<usize> = (0..context.metafits_context.num_baselines).collect();
 
-        let (jones_array, _) = context_to_jones_array(
-            &context,
-            &img_timestep_range,
-            &img_coarse_chan_range,
-            // img_baseline_idxs.as_slice(),
-            None,
-        );
+        let (jones_array, _) =
+            context_to_jones_array(&context, &img_timestep_range, &img_coarse_chan_range, None)
+                .unwrap();
 
         let jones_array = jones_array.mapv(TestJones::from);
 
@@ -709,13 +708,9 @@ mod tests {
         // let phase_centre_ha = phase_centre_ra.to_hadec(lst_rad);
         let tiles_xyz_geod = XyzGeodetic::get_tiles_mwa(&context.metafits_context);
 
-        let (jones_array, _) = context_to_jones_array(
-            &context,
-            &img_timestep_range,
-            &img_coarse_chan_range,
-            // img_baseline_idxs.as_slice(),
-            None,
-        );
+        let (jones_array, _) =
+            context_to_jones_array(&context, &img_timestep_range, &img_coarse_chan_range, None)
+                .unwrap();
 
         let jones_array = jones_array.mapv(TestJones::from);
 
@@ -814,6 +809,7 @@ mod tests {
             &img_timestep_range,
             &img_coarse_chan_range,
             None,
+            None,
         );
 
         let jones_array = jones_array.mapv(TestJones::from);
@@ -900,7 +896,8 @@ mod tests {
             &img_coarse_chan_range,
             // img_baseline_idxs.as_slice(),
             None,
-        );
+        )
+        .unwrap();
 
         let jones_array = jones_array.mapv(TestJones::from);
 
@@ -997,6 +994,7 @@ mod tests {
             &img_timestep_range,
             &img_coarse_chan_range,
             None,
+            None,
         );
         let jones_array = jones_array.mapv(TestJones::from);
         // there should be no difference in baseline 0
