@@ -31,7 +31,6 @@ use marlu::{
         get_required_fits_key,
     },
 };
-// use rayon::prelude::*;
 use regex::Regex;
 use std::collections::BTreeMap;
 use std::path::Path;
@@ -330,9 +329,7 @@ impl FlagFileSet {
             gpubox_ids,
             flag_array.axis_chunks_iter(Axis(1), fine_chans_per_coarse),
             write_progress
-        )
-        .into_iter()
-        {
+        ) {
             match self.gpubox_fptrs.get_mut(&gpubox_id) {
                 None => {
                     return Err(InvalidGpuBox {
@@ -825,8 +822,15 @@ mod tests {
             .map(|chan| chan.gpubox_number)
             .collect::<Vec<_>>();
 
-        let mut flag_array =
-            init_flag_array(&context, &img_timestep_range, &img_coarse_chan_range, None);
+        let mut flag_array = init_flag_array(
+            &context,
+            &img_timestep_range,
+            &img_coarse_chan_range,
+            None,
+            None,
+            None,
+            None,
+        );
 
         let mut idx = 0;
 
